# frozen_string_literal: true

require 'spec_helper'

RSpec.describe Upload do
  include EE::GeoHelpers
  using RSpec::Parameterized::TableSyntax

  describe '.replicables_for_geo_node' do
    # Selective sync is configured relative to the upload's model. Take care not
    # to specify a model_factory that contradicts factory.
    #
    # Permutations of sync_object_storage combined with object-stored-uploads
    # are tested in code, because the logic is simple, and to do it in the table
    # would quadruple its size and have too much duplication.
    where(:selective_sync_namespaces, :selective_sync_shards, :factory, :model_factory, :is_upload_included) do
      nil                  | nil            | [:upload]                    | [:project]               | true
      nil                  | nil            | [:upload, :issuable_upload]  | [:project]               | true
      nil                  | nil            | [:upload, :namespace_upload] | [:group]                 | true
      nil                  | nil            | [:upload, :favicon_upload]   | [:appearance]            | true
      # selective sync by shard
      nil                  | :model         | [:upload]                    | [:project]               | true
      nil                  | :other         | [:upload]                    | [:project]               | false
      nil                  | :model_project | [:upload, :namespace_upload] | [:group]                 | true
      nil                  | :other         | [:upload, :namespace_upload] | [:group]                 | false
      nil                  | :other         | [:upload, :favicon_upload]   | [:appearance]            | true
      # selective sync by namespace
      :model_parent        | nil            | [:upload]                    | [:project]               | true
      :model_parent_parent | nil            | [:upload]                    | [:project, :in_subgroup] | true
      :model               | nil            | [:upload, :namespace_upload] | [:group]                 | true
      :model_parent        | nil            | [:upload, :namespace_upload] | [:group, :nested]        | true
      :other               | nil            | [:upload]                    | [:project]               | false
      :other               | nil            | [:upload]                    | [:project, :in_subgroup] | false
      :other               | nil            | [:upload, :namespace_upload] | [:group]                 | false
      :other               | nil            | [:upload, :namespace_upload] | [:group, :nested]        | false
      :other               | nil            | [:upload, :favicon_upload]   | [:appearance]            | true
    end

    with_them do
      subject(:upload_included) { described_class.replicables_for_geo_node.include?(upload) }

      let(:model) { create(*model_factory) }
      let(:node) { create_geo_node(model, sync_object_storage: sync_object_storage) }

      before do
        stub_current_geo_node(node)
      end

      context 'when sync object storage is enabled' do
        let(:sync_object_storage) { true }

        context 'when the upload is locally stored' do
          let(:upload) { create(*factory, model: model) }

          it { is_expected.to eq(is_upload_included) }
        end

        context 'when the upload is object stored' do
          let(:upload) { create(*factory, :object_storage, model: model) }

          it { is_expected.to eq(is_upload_included) }
        end
      end

      context 'when sync object storage is disabled' do
        let(:sync_object_storage) { false }

        context 'when the upload is locally stored' do
          let(:upload) { create(*factory, model: model) }

          it { is_expected.to eq(is_upload_included) }
        end

        context 'when the upload is object stored' do
          let(:upload) { create(*factory, :object_storage, model: model) }

          it { is_expected.to be_falsey }
        end
      end
    end

    def create_geo_node(model, sync_object_storage:)
      node = build(:geo_node)

      if selective_sync_namespaces
        node.selective_sync_type = 'namespaces'
      elsif selective_sync_shards
        node.selective_sync_type = 'shards'
      end

      case selective_sync_namespaces
      when :model
        node.namespaces = [model]
      when :model_parent
        node.namespaces = [model.parent]
      when :model_parent_parent
        node.namespaces = [model.parent.parent]
      when :other
        node.namespaces = [create(:group)]
      end

      case selective_sync_shards
      when :model
        node.selective_sync_shards = [model.repository_storage]
      when :model_project
        project = create(:project, namespace: model)
        node.selective_sync_shards = [project.repository_storage]
      when :other
        node.selective_sync_shards = ['other_shard_name']
      end

      node.sync_object_storage = sync_object_storage

      node.save!
      node
    end
  end

  describe '#destroy' do
    subject { create(:upload, checksum: '8710d2c16809c79fee211a9693b64038a8aae99561bc86ce98a9b46b45677fe4') }

    context 'when running in a Geo primary node' do
      let_it_be(:primary) { create(:geo_node, :primary) }
      let_it_be(:secondary) { create(:geo_node) }

      it 'logs an event to the Geo event log' do
        stub_current_geo_node(primary)

        expect { subject.destroy }.to change(Geo::UploadDeletedEvent, :count).by(1)
      end
    end
  end
end
